@@ -13,6 +13,7 @@ import inspect
 import os
 from signal import signal, default_int_handler, SIGINT
 import sys
+import socket
 import time
 import uuid
 import warnings
@@ -127,6 +128,15 @@ class Kernel(SingletonConfigurable):
     # any links that should go in the help menu
     help_links = List()
 
+    # Experimental option to break in non-user code.
+    # The ipykernel source is in the call stack, so the user
+    # has to manipulate the step-over and step-into in a wize way.
+    debug_just_my_code = Bool(True,
+        help="""Set to False if you want to debug python standard and dependent libraries.
+        """
+    ).tag(config=True)
+
+    # track associations with current request
     # Private interface
 
     _darwin_app_nap = Bool(True,
@@ -671,7 +681,7 @@ class Kernel(SingletonConfigurable):
         self.log.debug("%s", reply_msg)
 
         if not silent and reply_msg['content']['status'] == 'error' and stop_on_error:
-            await self._abort_queues()
+            self._abort_queues()
 
     def do_execute(self, code, silent, store_history=True,
                    user_expressions=None, allow_stdin=False):
@@ -879,7 +889,9 @@ class Kernel(SingletonConfigurable):
             return None
 
     async def usage_request(self, stream, ident, parent):
-        reply_content = {}
+        reply_content = {
+            'hostname': socket.gethostname()
+        }
         if psutil is None:
             return reply_content
         current_process = psutil.Process()
@@ -974,13 +986,32 @@ class Kernel(SingletonConfigurable):
 
     _aborting = Bool(False)
 
-    async def _abort_queues(self):
-        self.shell_stream.flush()
+    def _abort_queues(self):
+        # while this flag is true,
+        # execute requests will be aborted
         self._aborting = True
-        def stop_aborting():
+        self.log.info("Aborting queue")
+
+        # flush streams, so all currently waiting messages
+        # are added to the queue
+        self.shell_stream.flush()
+
+        # Callback to signal that we are done aborting
+        # dispatch functions _must_ be async
+        async def stop_aborting():
             self.log.info("Finishing abort")
             self._aborting = False
-        asyncio.get_event_loop().call_later(self.stop_on_error_timeout, stop_aborting)
+
+        # put the stop-aborting event on the message queue
+        # so that all messages already waiting in the queue are aborted
+        # before we reset the flag
+        schedule_stop_aborting = partial(self.schedule_dispatch, stop_aborting)
+
+        # if we have a delay, give messages this long to arrive on the queue
+        # before we stop aborting requests
+        asyncio.get_event_loop().call_later(
+            self.stop_on_error_timeout, schedule_stop_aborting
+        )
 
     def _send_abort_reply(self, stream, msg, idents):
         """Send a reply to an aborted request"""
