@@ -8,8 +8,9 @@ import sys
 from distutils.version import LooseVersion as V
 from queue import Empty
 
-import nose.tools as nt
-from nose.plugins.skip import SkipTest
+import pytest
+
+import jupyter_client
 
 from traitlets import (
     HasTraits, TraitError, Bool, Unicode, Dict, Integer, List, Enum
@@ -61,7 +62,7 @@ class Version(Unicode):
         self.min = kwargs.pop('min', None)
         self.max = kwargs.pop('max', None)
         kwargs['default_value'] = self.min
-        super(Version, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def validate(self, obj, value):
         if self.min and V(value) < V(self.min):
@@ -78,7 +79,7 @@ class RMessage(Reference):
     content = Dict()
 
     def check(self, d):
-        super(RMessage, self).check(d)
+        super().check(d)
         RHeader().check(self.header)
         if self.parent_header:
             RHeader().check(self.parent_header)
@@ -288,21 +289,25 @@ def test_execute_silent():
     msg_id, reply = execute(code='x=1', silent=True)
 
     # flush status=idle
-    status = KC.iopub_channel.get_msg(timeout=TIMEOUT)
+    status = KC.get_iopub_msg(timeout=TIMEOUT)
     validate_message(status, 'status', msg_id)
     assert status['content']['execution_state'] == 'idle'
 
-    nt.assert_raises(Empty, KC.iopub_channel.get_msg, timeout=0.1)
+    with pytest.raises(Empty):
+        KC.get_iopub_msg(timeout=0.1)
+
     count = reply['execution_count']
 
     msg_id, reply = execute(code='x=2', silent=True)
 
     # flush status=idle
-    status = KC.iopub_channel.get_msg(timeout=TIMEOUT)
+    status = KC.get_iopub_msg(timeout=TIMEOUT)
     validate_message(status, 'status', msg_id)
     assert status['content']['execution_state'] == 'idle'
 
-    nt.assert_raises(Empty, KC.iopub_channel.get_msg, timeout=0.1)
+    with pytest.raises(Empty):
+        KC.get_iopub_msg(timeout=0.1)
+
     count_2 = reply['execution_count']
     assert count_2 == count
 
@@ -314,7 +319,7 @@ def test_execute_error():
     assert reply['status'] == 'error'
     assert reply['ename'] == 'ZeroDivisionError'
 
-    error = KC.iopub_channel.get_msg(timeout=TIMEOUT)
+    error = KC.get_iopub_msg(timeout=TIMEOUT)
     validate_message(error, 'error', msg_id)
 
 
@@ -322,35 +327,44 @@ def test_execute_inc():
     """execute request should increment execution_count"""
     flush_channels()
 
-    msg_id, reply = execute(code='x=1')
-    count = reply['execution_count']
+    _, reply = execute(code="x=1")
+    count = reply["execution_count"]
 
     flush_channels()
 
-    msg_id, reply = execute(code='x=2')
-    count_2 = reply['execution_count']
-    assert count_2 == count+1
+    _, reply = execute(code="x=2")
+    count_2 = reply["execution_count"]
+    assert count_2 == count + 1
+
 
 def test_execute_stop_on_error():
     """execute request should not abort execution queue with stop_on_error False"""
     flush_channels()
 
-    fail = '\n'.join([
-        # sleep to ensure subsequent message is waiting in the queue to be aborted
-        'import time',
-        'time.sleep(0.5)',
-        'raise ValueError',
-    ])
+    fail = "\n".join(
+        [
+            # sleep to ensure subsequent message is waiting in the queue to be aborted
+            # async sleep to ensure coroutines are processing while this happens
+            "import asyncio",
+            "await asyncio.sleep(1)",
+            "raise ValueError()",
+        ]
+    )
     KC.execute(code=fail)
-    msg_id = KC.execute(code='print("Hello")')
-    KC.get_shell_msg(timeout=TIMEOUT)
+    KC.execute(code='print("Hello")')
+    KC.execute(code='print("world")')
+    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    print(reply)
+    reply = KC.get_shell_msg(timeout=TIMEOUT)
+    assert reply["content"]["status"] == "aborted"
+    # second message, too
     reply = KC.get_shell_msg(timeout=TIMEOUT)
     assert reply['content']['status'] == 'aborted'
 
     flush_channels()
 
     KC.execute(code=fail, stop_on_error=False)
-    msg_id = KC.execute(code='print("Hello")')
+    KC.execute(code='print("Hello")')
     KC.get_shell_msg(timeout=TIMEOUT)
     reply = KC.get_shell_msg(timeout=TIMEOUT)
     assert reply['content']['status'] == 'ok'
@@ -385,11 +399,11 @@ def test_user_expressions():
 
     msg_id, reply = execute(code='x=1', user_expressions=dict(foo='x+1'))
     user_expressions = reply['user_expressions']
-    nt.assert_equal(user_expressions, {'foo': {
+    assert user_expressions == {'foo': {
         'status': 'ok',
         'data': {'text/plain': '2'},
         'metadata': {},
-    }})
+    }}
 
 
 def test_user_expressions_fail():
@@ -482,10 +496,12 @@ def test_connect_request():
     validate_message(reply, 'connect_reply', msg_id)
 
 
+@pytest.mark.skipif(
+    jupyter_client.version_info < (5, 0),
+    reason="earlier Jupyter Client don't have comm_info",
+)
 def test_comm_info_request():
     flush_channels()
-    if not hasattr(KC, 'comm_info'):
-        raise SkipTest()
     msg_id = KC.comm_info()
     reply = get_reply(KC, msg_id, TIMEOUT)
     validate_message(reply, 'comm_info_reply', msg_id)
@@ -519,8 +535,8 @@ def test_is_complete():
 def test_history_range():
     flush_channels()
 
-    msg_id_exec = KC.execute(code='x=1', store_history = True)
-    reply_exec = KC.get_shell_msg(timeout=TIMEOUT)
+    KC.execute(code="x=1", store_history=True)
+    KC.get_shell_msg(timeout=TIMEOUT)
 
     msg_id = KC.history(hist_access_type = 'range', raw = True, output = True, start = 1, stop = 2, session = 0)
     reply = get_reply(KC, msg_id, TIMEOUT)
@@ -531,8 +547,8 @@ def test_history_range():
 def test_history_tail():
     flush_channels()
 
-    msg_id_exec = KC.execute(code='x=1', store_history = True)
-    reply_exec = KC.get_shell_msg(timeout=TIMEOUT)
+    KC.execute(code="x=1", store_history=True)
+    KC.get_shell_msg(timeout=TIMEOUT)
 
     msg_id = KC.history(hist_access_type = 'tail', raw = True, output = True, n = 1, session = 0)
     reply = get_reply(KC, msg_id, TIMEOUT)
@@ -543,8 +559,8 @@ def test_history_tail():
 def test_history_search():
     flush_channels()
 
-    msg_id_exec = KC.execute(code='x=1', store_history = True)
-    reply_exec = KC.get_shell_msg(timeout=TIMEOUT)
+    KC.execute(code="x=1", store_history=True)
+    KC.get_shell_msg(timeout=TIMEOUT)
 
     msg_id = KC.history(hist_access_type = 'search', raw = True, output = True, n = 1, pattern = '*', session = 0)
     reply = get_reply(KC, msg_id, TIMEOUT)
@@ -560,7 +576,7 @@ def test_stream():
 
     msg_id, reply = execute("print('hi')")
 
-    stdout = KC.iopub_channel.get_msg(timeout=TIMEOUT)
+    stdout = KC.get_iopub_msg(timeout=TIMEOUT)
     validate_message(stdout, 'stream', msg_id)
     content = stdout['content']
     assert content['text'] == 'hi\n'
@@ -571,7 +587,7 @@ def test_display_data():
 
     msg_id, reply = execute("from IPython.display import display; display(1)")
 
-    display = KC.iopub_channel.get_msg(timeout=TIMEOUT)
+    display = KC.get_iopub_msg(timeout=TIMEOUT)
     validate_message(display, 'display_data', parent=msg_id)
     data = display['content']['data']
     assert data['text/plain'] == '1'
